/* eslint-disable */
require('script-loader!file-saver'); //For saving files
require('./Blob.js'); //For binary conversion
require('script-loader!xlsx/dist/xlsx.core.min'); //xlsx core

function generateArray(table) {
	var out = [];
	var rows = table.querySelectorAll('tr');
	var ranges = [];
	for (var R = 0; R < rows.length; ++R) {
		var outRow = [];
		var row = rows[R];
		var columns = row.querySelectorAll('td');
		for (var C = 0; C < columns.length; ++C) {
			var cell = columns[C];
			var colspan = cell.getAttribute('colspan');
			var rowspan = cell.getAttribute('rowspan');
			var cellValue = cell.innerText;
			if (cellValue !== "" && cellValue == +cellValue) cellValue = +cellValue;

			//Skip ranges
			ranges.forEach(function(range) {
				if (R >= range.s.r && R <= range.e.r && outRow.length >= range.s.c && outRow.length <= range.e
					.c) {
					for (var i = 0; i <= range.e.c - range.s.c; ++i) outRow.push(null);
				}
			});

			//Handle Row Span
			if (rowspan || colspan) {
				rowspan = rowspan || 1;
				colspan = colspan || 1;
				ranges.push({
					s: {
						r: R,
						c: outRow.length
					},
					e: {
						r: R + rowspan - 1,
						c: outRow.length + colspan - 1
					}
				});
			};

			//Handle Value
			outRow.push(cellValue !== "" ? cellValue : null);

			//Handle Colspan
			if (colspan)
				for (var k = 0; k < colspan - 1; ++k) outRow.push(null);
		}
		out.push(outRow);
	}
	return [out, ranges];
};

function datenum(v, date1904) {
	if (date1904) v += 1462;
	var epoch = Date.parse(v);
	return (epoch - new Date(Date.UTC(1899, 11, 30))) / (24 * 60 * 60 * 1000);
}

function sheet_from_array_of_arrays(data, opts) {
	var ws = {};
	var range = {
		s: {
			c: 10000000,
			r: 10000000
		},
		e: {
			c: 0,
			r: 0
		}
	};
	for (var R = 0; R != data.length; ++R) {
		for (var C = 0; C != data[R].length; ++C) {
			if (range.s.r > R) range.s.r = R;
			if (range.s.c > C) range.s.c = C;
			if (range.e.r < R) range.e.r = R;
			if (range.e.c < C) range.e.c = C;
			var cell = {
				v: data[R][C]
			};
			if (cell.v == null) continue;
			var cell_ref = XLSX.utils.encode_cell({
				c: C,
				r: R
			});

			if (typeof cell.v === 'number') cell.t = 'n';
			else if (typeof cell.v === 'boolean') cell.t = 'b';
			else if (cell.v instanceof Date) {
				cell.t = 'n';
				cell.z = XLSX.SSF._table[14];
				cell.v = datenum(cell.v);
			} else cell.t = 's';

			ws[cell_ref] = cell;
		}
	}
	if (range.s.c < 10000000) ws['!ref'] = XLSX.utils.encode_range(range);
	return ws;
}

function Workbook() {
	if (!(this instanceof Workbook)) return new Workbook();
	this.SheetNames = [];
	this.Sheets = {};
}

function s2ab(s) {
	var buf = new ArrayBuffer(s.length);
	var view = new Uint8Array(buf);
	for (var i = 0; i != s.length; ++i) view[i] = s.charCodeAt(i) & 0xFF;
	return buf;
}

export function export_table_to_excel(id) {
	var theTable = document.getElementById(id);
	var oo = generateArray(theTable);
	var ranges = oo[1];

	/* original data */
	var data = oo[0];
	var ws_name = "SheetJS";
	// console.log(data);

	var wb = new Workbook(),
		ws = sheet_from_array_of_arrays(data);

	/* add ranges to worksheet */
	// ws['!cols'] = ['apple', 'banan'];
	ws['!merges'] = ranges;

	/* add worksheet to workbook */
	wb.SheetNames.push(ws_name);
	wb.Sheets[ws_name] = ws;

	var wbout = XLSX.write(wb, {
		bookType: 'xlsx',
		bookSST: false,
		type: 'binary'
	});

	saveAs(new Blob([s2ab(wbout)], {
		type: "application/octet-stream"
	}), "test.xlsx")
}

function formatJson(jsonData) {
	console.log(jsonData)
}

export function export_json_to_excel(th, jsonData, defaultTitle) {

	var data = jsonData;

	// 添加标题
	for (var i = 0; i < th.length; i++) {
		data[i].unshift([th[i]])
	}

	// 这里是定义sheet的名称 有几个sheet就加几个
	var ws_name = [];
	th.forEach(item => {
		ws_name.push(item);
	});

	var wb = new Workbook(),
		ws = [];
	//数据转换
	for (var j = 0; j < th.length; j++) {
		ws.push(sheet_from_array_of_arrays(data[j]))
	}

	//生成多个sheet
	for (var k = 0; k < th.length; k++) {
		wb.SheetNames.push(ws_name[k])
		wb.Sheets[ws_name[k]] = ws[k]
	}

	var wbout = XLSX.write(wb, {
		bookType: 'xlsx',
		bookSST: false,
		type: 'binary'
	});

	var title = defaultTitle || '列表'
	saveAs(new Blob([s2ab(wbout)], {
		type: "application/octet-stream"
	}), title + ".xlsx");
}

export function export_json_to_excel_merges({
	multiHeader = [],
	header,
	data,
	filename,
	merges = [],
	autoWidth = true,
	bookType = 'xlsx'
} = {}) {
	/* original data */
	filename = filename || '列表';
	data = [...data]
	data.unshift(header);
	for (let i = multiHeader.length - 1; i > -1; i--) {
		data.unshift(multiHeader[i])
	}
	
	var ws_name = "Sheet1";
	var wb = new Workbook(),
		ws = sheet_from_array_of_arrays(data);
	
	/* add worksheet to workbook */
	wb.SheetNames.push(ws_name);
	wb.Sheets[ws_name] = ws;
	
	// 单元格样式定义
	var dataInfo = wb.Sheets[wb.SheetNames[0]];
	const tableTitleFont = {
		font: {
		  name: "微软雅黑",
		  sz: 18,
		},
		alignment: {
		  horizontal: "center",
		  vertical: "center",
		},
		bold: true
	}
	const tableColumnFont = {
		font: {
		  name: "微软雅黑",
		  sz: 11,
		},
		alignment: {
			horizontal: "center",
			vertical: "center",
			wrapText: 1,
		}
	}
	dataInfo["A1"].s = tableTitleFont
	// dataInfo["A2"].s = tableTitleFont
	// 从第2行开始
	let regexColumn = /[A-Z]2$/;
	for (var b in dataInfo) {
	  if (regexColumn.test(b)) {
	    dataInfo[b].s = tableColumnFont;
	  }
	}
	
	// 合并单元格
	if (merges.length > 0) {
		if (!ws['!merges']) ws['!merges'] = [];
		merges.forEach(item => {
			ws['!merges'].push(XLSX.utils.decode_range(item))
		})
	}
	
	// 自动计算单元格宽度
	if (autoWidth) {
		let colWidths = [];
		// 计算每一列的所有单元格宽度
		// 先遍历行
		data.forEach((row) => {
			// 列序号
			let index = 0
			// 遍历列
			for (const key in row) {
				if (colWidths[index] == null) colWidths[index] = []
				switch (typeof row[key]) {
					case 'string':
					case 'number':
					case 'boolean':
						colWidths[index].push(getCellWidth(row[key]))
						break
					case 'object':
					case 'function':
						colWidths[index].push(0)
						break
				}
				index++
			}
		})
		ws['!cols'] = [];
		colWidths.forEach((widths, index) => {
			// 计算列头的宽度
			widths.push(getCellWidth(header[index]))
			// 设置最大值为列宽
			ws['!cols'].push({
				wch: Math.max(...widths)
			})
		})
	}
	
	var wbout = XLSX.write(wb, {
		bookType: bookType,
		bookSST: false,
		type: 'binary'
	});
	
	saveAs(new Blob([s2ab(wbout)], {
		type: "application/octet-stream"
	}), `${filename}.${bookType}`);
}

export function getCellWidth(value) {
	if (value == null) {
		return 10
	} else if (value.toString().charCodeAt(0) > 255) {
		// 判断是否包含中文
		let length = value.toString().length * 2
		if (length > 60) {
			length = length - 40
			//这里的宽度可以自己设定，在前面设置wrapText: 1可以在单元格内换行
		}
		return length + 2
	} else {
		return value.toString().length * 1.2
	}
}
