// 定义运行状态字典
export const runStatDict = {
	0: {
		label: '离线',
		color: 'lightgray'
	},
	1: {
		label: '在线',
		color: 'yellowgreen'
	},
	2: {
		label: '未关联',
		color: 'lightgray'
	}
}

// 定义运行模式字典
export const runModeDict = {
	0: {
		label: '无',
		color: 'lightgray'
	},
	1: {
		label: '制冷',
		color: 'darkgreen'
	},
	2: {
		label: '采暖',
		color: 'orange'
	},
	3: {
		label: '除湿',
		color: 'rgb(182, 172, 195)'
	},
	4: {
		label: '预热',
		color: 'yellow'
	},
	5: {
		label: '通风',
		color: 'rgb(182, 174, 227)'
	}
}

// 定义连接状态字典
export const connectStatDict = {
	0: {
		label: '异常',
		color: 'yellow'
	},
	1: {
		label: '正常',
		color: 'yellowgreen'
	}
}

// 定义故障字典
export const faultDict = {
	0: {
		label: '无',
		color: 'yellowgreen'
	},
	1: {
		label: '有',
		color: 'yellow'
	}
}

// 定义换热站水泵状态字典
export const hexPumpStatDict = {
	0: {
		label: '停止',
		color: 'yellow'
	},
	1: {
		label: '运行',
		color: 'yellowgreen'
	}
}

// 运行状态选项定义
export const runStatOpts = [{
		label: '在线',
		value: '1'
	},
	{
		label: '离线',
		value: '0'
	}
]

// 运行模式选项定义
export const runModeOpts = [{
		label: '制冷',
		value: '1'
	},
	{
		label: '采暖',
		value: '2'
	},
	{
		label: '除湿',
		value: '3'
	},
	{
		label: '预热',
		value: '4'
	},
	{
		label: '通风',
		value: '5'
	}
]

// 定义弹窗的主机列
export const hostCols = [{
		prop: 'host_order',
		label: '编号',
		width: 60
	},
	{
		prop: 'host_connect_status',
		label: '通讯状态'
	},
	{
		prop: 'host_fault',
		label: '故障'
	},
	{
		prop: 'host_su_temp',
		label: '供水水温（℃）'
	},
	{
		prop: 'host_re_temp',
		label: '回水水温（℃）'
	},
	{
		prop: 'host_temp_diff',
		label: '温差（℃）'
	}
]

// 新风机列
export const newTrendCols = [{
		prop: 'nt_order',
		label: '编号',
		width: 60
	},
	{
		prop: 'nt_connect_status',
		label: '通讯状态'
	},
	{
		prop: 'nt_fault',
		label: '故障'
	},
	{
		prop: 'nt_in_temp',
		label: '送风温度（℃）'
	},
	{
		prop: 'nt_in_humidity',
		label: '送风湿度（℃）'
	},
	{
		prop: 'nt_dew_point',
		label: '露点（℃）'
	}
]

// 换热站列
export const heatExchangeCols = [{
		prop: 'hex_order',
		label: '编号',
		width: 60
	},
	{
		prop: 'hex_connect_status',
		label: '通讯状态'
	},
	{
		prop: 'hex_fault',
		label: '故障'
	},
	{
		prop: 'hex_su_temp',
		label: '供水水温（℃）'
	},
	{
		prop: 'hex_re_temp',
		label: '回水水温（℃）'
	},
	{
		prop: 'hex_temp_diff',
		label: '温差（℃）'
	},
	{
		prop: 'hex_pump_status',
		label: '水泵状态'
	}
]

// 末端列
export const endCols = [{
		prop: 'end_order',
		label: '编号',
		width: 60
	},
	{
		prop: 'end_exception_num',
		label: '通讯异常数量'
	},
	{
		prop: 'end_min_temp',
		label: '最低室内温度（℃）'
	},
	{
		prop: 'end_min_humidity',
		label: '最低室内湿度（%）'
	},
	{
		prop: 'end_min_dew_point',
		label: '最低室内露点（℃）'
	},
	{
		prop: 'end_max_temp',
		label: '最高室内温度（℃）'
	},
	{
		prop: 'end_max_humidity',
		label: '最高室内湿度（%）'
	},
	{
		prop: 'end_max_dew_point',
		label: '最高室内露点（℃）'
	}
]

/**
 * 函数：通过数组描述执行表达式，得到变量对应的颜色
 */
export function getArgsColor(argsList, runMode, val) {
	let evalString = ""
	if (argsList.length <= 0) {
		return evalString
	}
	
	var color = ''
	// 先找出条件中有模式的数组跟无模式的数组
	// 先拼接带有模式的数组, 再组合一下无模式的数组
	let runModeArgs = argsList.filter(args => args.RunMode > 0)
	let noneRunModeArgs = argsList.filter(args => args.RunMode <= 0)
	runModeArgs.forEach(args => {
		evalString += `if (runMode==${args.RunMode}) {color=val${args.Action}${args.Threshold}?'yellow':'yellowgreen'} else `
	})
	if (noneRunModeArgs.length > 0) {
		let tmp = 'if (';
		noneRunModeArgs.forEach(args => {
			tmp += `val${args.Action}${args.Threshold}`
			tmp += '||'
		})
		tmp = tmp.substring(0, tmp.length - 2)
		tmp += `) {color='yellow'} else {color='yellowgreen'}`
		evalString += tmp
	} else {
		evalString = evalString.substring(0, evalString.length - 5)
	}
	
	// if (argsList[0].En === 'nt_dew_point') {
	// 	console.log(">>> argsList: ", argsList, runMode, val)
	// 	console.log(">>> evalString: ", evalString)
	// }
	
	// 执行命令
	eval(evalString)
	return color
}
