import { Export2ExcelMerge } from './export_excel.js'
import { listProjects } from './api'
import { runStatDict, runModeDict, connectStatDict, faultDict, hexPumpStatDict } from './common.js'

var config = {
	rowDatas: [],  // 数据
	page: 1,  // 当前页
	load: 0,  // 已加载数量
}

function reset() {
	config.rowDatas = []
	config.page = 1
	config.load = 0
}

export function exportToExcel(datas, fn) {
	/*
	reset()
	_listAllProjects(function(finished) {
		// Excel文件名
		let defaultFilename = 'LINKSEE-智能报表'
		// 多级表头
		const multiHeader = [[
			'项目名称', '盒子名称', '在线状态', '运行模式',
			'主机', '', '', '', '', '',
			'新风机', '', '', '', '', '',
			'换热站', '', '', '', '', '', '',
			'末端', '', '', '', '', '', '', ''
		]]
		// 二级表头
		let header = [
			'项目名称', '盒子名称', '在线状态', '运行模式',
			'编号', '通讯状态', '故障', '供水水温（℃）', '回水水温（℃）', '温差',
			'编号', '通讯状态', '故障', '送风温度（℃）', '送风湿度（%）', '露点（℃）',
			'编号', '通讯状态', '故障', '供水水温（℃）', '回水水温（℃）', '温差', '水泵状态',
			'编号', '通讯异常数量', '最低室内温度（℃）', '最低室内湿度（%）', '最低室内露点（℃）', '最高室内温度（℃）', '最高室内湿度（%）', '最高室内露点（℃）'
		]
		// 合并项
		const merges = ['A1:A2', 'B1:B2', 'C1:C2', 'D1:D2', 'E1:J1', 'K1:P1', 'Q1:W1', 'X1:AE1']
		// 数据整理
		// let tableDatas = []
		// config.rowDatas.forEach(row => {
		// 	tableDatas.push(row)
		// })
		Export2ExcelMerge({
			multiHeader, 
			header: header, 
			data: config.rowDatas, 
			filename: defaultFilename, 
			merges
		})
		fn()
	})
	*/
 
	// Excel文件名
	let defaultFilename = 'LINKSEE-智能报表'
	// 多级表头
	const multiHeader = [[
		'项目名称', '盒子名称', '在线状态', '运行模式',
		'主机', '', '', '', '', '',
		'新风机', '', '', '', '', '',
		'换热站', '', '', '', '', '', '',
		'末端', '', '', '', '', '', '', ''
	]]
	// 二级表头
	let header = [
		'项目名称', '盒子名称', '在线状态', '运行模式',
		'编号', '通讯状态', '故障', '供水水温（℃）', '回水水温（℃）', '温差',
		'编号', '通讯状态', '故障', '送风温度（℃）', '送风湿度（%）', '露点（℃）',
		'编号', '通讯状态', '故障', '供水水温（℃）', '回水水温（℃）', '温差', '水泵状态',
		'编号', '通讯异常数量', '最低室内温度（℃）', '最低室内湿度（%）', '最低室内露点（℃）', '最高室内温度（℃）', '最高室内湿度（%）', '最高室内露点（℃）'
	]
	// 合并项
	const merges = ['A1:A2', 'B1:B2', 'C1:C2', 'D1:D2', 'E1:J1', 'K1:P1', 'Q1:W1', 'X1:AE1']
	// 数据
	let data = _transferDataToXlsx(datas);
	Export2ExcelMerge({
		multiHeader, 
		header: header, 
		data: data, 
		filename: defaultFilename, 
		merges
	})
	fn()
}

function _listAllProjects(fn) {
	let queryParams = { page_index: config.page, page_size: 20 }
	listProjects(queryParams).then(resp => {
		let result = resp.data.result || {}
		let datas = result.data || []
		let total = result.count
		config.load += datas.length;
		if (reset <= 1) {
			config.rowDatas = _transferDataToXlsx(datas)
		} else {
			config.rowDatas = config.rowDatas.concat(_transferDataToXlsx(datas))
		}
		// if (config.load < total && datas.length > 0) {
		if (config.page > 1) {
			setTimeout(() => {
				config.page += 1
				_listAllProjects(fn)
			}, 50)
		} else {
			fn(true)
		}
	})
}

function _transferDataToXlsx(projects) {
	let tableData = []
	projects.forEach(p => {
		if (Array.isArray(p.boxList)) {
			p.boxList.forEach(b => {
				// 继续判断有多少单机，主机、新风机、换热站、末端单机数量取最大值
				let maxCnt = _calcMaxStandaloneCnt(b)
				for (let i = 0; i < maxCnt; i++) {
					let hpStandalone = b.hostCtrl && i < b.hostCtrl.length ? b.hostCtrl[i] : undefined
					let dhStandalone = b.newTrendCtrl && i < b.newTrendCtrl.length ? b.newTrendCtrl[i] : undefined
					let hexStandalone = b.hex && i < b.hex.length ? b.hex[i] : undefined
					let cbStandalone = b.end && i < b.end.length ? b.end[i] : undefined
					let row = [
						p.projectName,
						b.boxName,
						runStatDict[b.runStatus] ? runStatDict[b.runStatus].label : '/',
						runModeDict[b.runMode] ? runModeDict[b.runMode].label : '/',
						hpStandalone ? hpStandalone.host_order : '',
						hpStandalone ? _getConnectState(hpStandalone.host_connect_status) : '',
						hpStandalone ? _getFault(hpStandalone.host_fault) : '',
						hpStandalone ? hpStandalone.host_su_temp : '',
						hpStandalone ? hpStandalone.host_re_temp : '',
						hpStandalone ? hpStandalone.host_temp_diff : '',
						dhStandalone ? dhStandalone.nt_order : '',
						dhStandalone ? _getConnectState(dhStandalone.nt_connect_status) : '',
						dhStandalone ? _getFault(dhStandalone.nt_fault) : '',
						dhStandalone ? dhStandalone.nt_in_temp : '',
						dhStandalone ? dhStandalone.nt_in_humidity : '',
						dhStandalone ? dhStandalone.nt_dew_point : '',
						hexStandalone ? hexStandalone.hex_order : '',
						hexStandalone ? _getConnectState(hexStandalone.hex_connect_status) : '',
						hexStandalone ? _getFault(hexStandalone.hex_fault) : '',
						hexStandalone ? hexStandalone.hex_su_temp : '',
						hexStandalone ? hexStandalone.hex_re_temp : '',
						hexStandalone ? hexStandalone.hex_temp_diff : '',
						hexStandalone ? _getHexPumpState(hexStandalone.hex_pump_status) : '',
						cbStandalone ? cbStandalone.end_order : '',
						cbStandalone ? cbStandalone.end_exception_num : '',
						cbStandalone ? cbStandalone.end_min_temp : '',
						cbStandalone ? cbStandalone.end_min_humidity : '',
						cbStandalone ? cbStandalone.end_min_dew_point : '',
						cbStandalone ? cbStandalone.end_max_temp : '',
						cbStandalone ? cbStandalone.end_max_humidity : '',
						cbStandalone ? cbStandalone.end_max_dew_point : '',
					]
					tableData.push(row)
				}
			})
		}
	})
	return tableData
}

// 取最大单机数
function _calcMaxStandaloneCnt(box) {
	let hpStandaloneCnt = box.hostCtrl ? box.hostCtrl.length : 0
	let dhStandaloneCnt = box.newTrendCtrl ? box.newTrendCtrl.length : 0
	let hexStandaloneCnt = box.hex ? box.hex.length : 0
	let cbStandalineCnt = box.end ? box.end.length : 0
	return Math.max(hpStandaloneCnt, dhStandaloneCnt, hexStandaloneCnt, cbStandalineCnt)
}

function _getConnectState(val) {
	return connectStatDict[val] ? connectStatDict[val].label : '/'
}

function _getFault(val) {
	return faultDict[val] ? faultDict[val].label : '/'
}

function _getHexPumpState(val) {
	return hexPumpStatDict[val] ? hexPumpStatDict[val].label : '/'
}